﻿/*
Summary: 
This sample program sends a *IDN? SCPI command to an 
instrument to get its identification string. The result will be 
displayed on a MS Windows Console.

Instructions:
1) Remove the sample VISA address shown in quotes below
"TCPIP0::255.255.255.255::inst0::INSTR" and replace
it with the VISA address of your instrument.  This must be done
within each of the 3 separate sections below.
NOTE: The VISA address of your instrument is provided by the
Keysight Connection Expert utility, which is part of the
Keysight IO Libraries. (see How to Get Started_CSharp.docx)
2) Save your change and then run this console application.

Prerequisites:
This program requires the Keysight VISA library, which is
part of the Keysight IO Libraries available for download at:
www.keysight.com/find/iolib

This sample program is provided by Keysight Technologies as-is.
Please feel free to contact us for Technical Support at:
phone: +1 800-829-4444
web: www.keysight.com

Program succesfully executes on:
Infiniium s/w ver 05.52.000x
Infiniivision-X s/w ver 04.06.xxx, 02.41.xxx, 02.37.xxx
FlexDCA s/w ver A.05.01.53
M8070A s/w ver 2.6.4.6 (failed with VXI-11 address)
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Ivi.Visa.Interop;

namespace InfiniivisionXGetIDN
{
    class Program
    {
        //Instruments VISA address as found in Keysight Connection Expert/NI-MAX
        const string instrAddress =
            "TCPIP0::255.255.255.255::inst0::INSTR";

        static void Main(string[] args)
        {
            string idnResult = "";

            // Get a handle to the default resource manager
            // Create an IO formatter
            ResourceManager rm = new ResourceManager();
            FormattedIO488 instrument = new FormattedIO488();

            try
            {
                // Open a connection to the instrument and bind 
                // it to the formatter. 
                instrument.IO = (IMessage)rm.Open(instrAddress,
                    AccessMode.NO_LOCK, 0, "");

                // Socket connections don't automatically add a
                // termination character so enable it in the interface.
                instrument.IO.TerminationCharacterEnabled = true;

                // There are other useful settings as well like setting 
                // the VISA timeout in miliseconds
                instrument.IO.Timeout = 3000;

                try
                {
                    //Send the *IDN? query SCPI command to the instrument
                    instrument.WriteString("*IDN?", true);

                    //Reading IDN string response from the instrument 
                    idnResult = instrument.ReadString();
                    Console.Out.WriteLine("Instrument *IDN? query returned: \n"
                        + idnResult);
                }

                finally
                {
                    try
                    {
                        // Close the connection to the instrument and free the reference
                        // to the session.
                        instrument.IO.Close();
                        instrument.IO = null;
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(instrument);
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                    }
                    catch (Exception errorClosing)
                    {
                        Console.Out.WriteLine("An error occured during closing; "
                            + errorClosing.Message);
                        Console.In.ReadLine();
                    }
                }

            }

            catch (Exception error)
            {
                Console.Out.WriteLine("An error occured; " + error.Message);
                Console.In.ReadLine();
            }
            Console.WriteLine("*** Press any key to exit ***");
            Console.ReadKey(true);
        }
    }
}
